# Copyright (c) OpenMMLab. All rights reserved.
from .masked_cross_entropy_loss import MaskedCrossEntropyLoss
from .masked_focal_loss import MaskedFocalLoss

__all__ = ['MaskedCrossEntropyLoss', 'MaskedFocalLoss']
