# Copyright (c) OpenMMLab. All rights reserved.
from . import extractors, heads, losses
from .extractors import *  # NOQA
from .heads import *  # NOQA
from .losses import *  # NOQA

__all__ = extractors.__all__ + heads.__all__ + losses.__all__
